#=========================================================================
#``Nowcasting PE ratio''
# by Andrii Babii, Ryan Ball, Eric Ghysels, Jonas Striaukas.
# File version: January, 13th, 2020
#-------------------------------------------------------------------------
# sort daily aggregate fin. markets data
#=========================================================================


rm(list=ls())
folder <- "working-directory"
setwd(folder)

var.names <- c("VIXCLS","DCOILWTICO","DBAA","DAAA","DGS10","T10Y3M","TEDRATE","T3MFF")  
date_range <- c(as.Date("1999-11-01"),as.Date("2017-07-31"))

require("quantmod") # version 0.4-16
require("alfred") # version 0.1.7

transform.dt <- function(x,tcode){
  t <- length(x)
  small <- 1e-9
  y <- rep(NA, t) 
  if(tcode==1) { # Level (i.e. no transformation): x(t)
    y = x
  }
  if(tcode==2){ # First difference: x(t)-x(t-1)
    y[-1] <- x[-1] - x[-t]
  }
  if(tcode==3){ #  Second difference: (x(t)-x(t-1))-(x(t-1)-x(t-2))
    y[-c(1,2)] <- x[-c(1,2)] - 2*x[-c(1,t)] + x[-c(t-1,t)]
  } 
  if(tcode==4){ #   Natural log: ln(x)
    x[abs(x)<small] <- NA
    y <- log(x)
  }  
  if(tcode==5){ #   First difference of natural log: ln(x)-ln(x-1)
    x[abs(x)<small] <- NA
    x <- log(x)
    y[-1] <- x[-1] - x[-t]
  }   
  if(tcode==6){ #   Second difference of natural log: ln(x)-2*ln(x-1)+ln(x-2)
    x[abs(x)<small] <- NA
    x <- log(x)
    y[-c(1,2)] <- x[-c(1,2)] - 2*x[-c(1,t)] + x[-c(t-1,t)]
  }    
  if(tcode==7){ #   First difference of percent change: (x(t)/x(t-1)-1)-(x(t-1)/x(t-2)-1)
    x[abs(x)<small] <- NA
    y.tmp <- y
    y.tmp[-1] <- (x[-1]-x[-t])/x[-t]
    y[-c(1,2)] <- y.tmp[-c(1,2)] - y.tmp[-c(1,t)] 
  } 
  return(y)
}

# download data from FRED:
daily_data <- NULL
for (j in 1:length(var.names)){
  idata <- NULL
  cat(paste0("downloading ", j, "-th series ", var.names[j], "\n"))
  while(is.null(idata)){
    idata <- suppressWarnings(tryCatch(get_fred_series(var.names[j],var.names[j],date_range[1],date_range[2]), error=function(e) NULL))
  }
  if (j == 1){
    daily_data <- cbind(daily_data,idata$date)
  }
  daily_data <- cbind(daily_data,idata[,2])
}
daily_data <- as.data.frame(daily_data)
daily_data[,1] <- as.Date(daily_data[,1])
colnames(daily_data) <- c("DATE",var.names)

# Transform the data:
daily_data$VXOCLS <- transform.dt(daily_data$VXOCLS,1) 
daily_data$DCOILWTICO <- transform.dt(daily_data$DCOILWTICO,6) 
daily_data$AAA_10Y <- transform.dt(daily_data$DAAA-daily_data$DGS10,1) 
daily_data$BAA_10Y <- transform.dt(daily_data$DBAA-daily_data$DGS10,1)  
daily_data$BAA_AAA <- transform.dt(daily_data$DBAA-daily_data$DAAA,1)
daily_data$DBAA <- NULL
daily_data$DAAA <- NULL
daily_data$DGS10 <- NULL
daily_data$T3MFF <- transform.dt(daily_data$T3MFF,1) 
daily_data$T10Y3M <- transform.dt(daily_data$T10Y3M,1) 
daily_data$TEDRATE <- transform.dt(daily_data$TEDRATE,1)


SNP500 <- read.csv("snp500ret.csv", header = TRUE, sep=',')
SNP500$DATE <- as.Date(SNP500$caldt)
SNP500$caldt <- NULL
# SNP500 taken from CRSP already in log returns
# match dates  SNP500 and daily_data (if no data => impute NA)

date1 <- daily_data$DATE
date2 <- SNP500$DATE

daily_data_update <- daily_data
for (i_date2 in 1:length(date2)){
  idate <- date2[i_date2]
 if (!idate%in%date1) {
   cat(paste0("added date with NA entries: ", idate,"\n"))
   ksplit <- max(which(daily_data_update$DATE<=idate))
   daily_data_update <- rbind(daily_data_update[1:ksplit, ], 
                              data.frame(DATE=idate, VIXCLS=NA, DCOILWTICO=NA, T10Y3M=NA, TEDRATE=NA, AAA_10Y=NA, BAA_10Y=NA, BAA_AAA=NA, T3MFF=NA),
                              daily_data_update[-c(1:ksplit), ])
 }
}
daily_data <- daily_data_update
SNP500_update <- SNP500
for (i_date1 in 1:length(date1)){
  idate <- date1[i_date1]
  if (!idate%in%date2) {
    cat(paste0("added date with NA entries: ", idate,"\n"))
    ksplit <- max(which(SNP500_update$DATE<=idate))
    SNP500_update <- rbind(SNP500_update[1:ksplit, ], 
                               data.frame(DATE=idate, sprtrn=NA),
                               SNP500_update[-c(1:ksplit), ])
  }
}
if(sum(SNP500_update$DATE%in%daily_data$DATE)==dim(daily_data)[1]){
  daily_data$SNP500 <- SNP500_update$sprtrn
} else {
  stop("data dont match")
}
daily_data <- daily_data[daily_data$DATE<as.Date("2017-08-01"),]
save(daily_data,file="daily_data.RData")
