rm(list=ls())
# ------- SET WORKING DIRECTORY ------- #
if (! "rstudioapi" %in% installed.packages() ) install.packages("rstudioapi")
wd <- dirname(rstudioapi::getSourceEditorContext()$path)
setwd(wd)

# functions
monthEnd <- function(x){
  lubridate::ceiling_date(x, "month") - lubridate::days(1)
}

date_range <- c(as.Date("1999-11-01"),as.Date("2017-07-31"))

# three firm-level daily crsp files of returns for convenience
returns_daily1 <- read.csv("RETURNS_DAILY.csv",header = TRUE)
returns_daily2 <- read.csv("RETURNS_DAILY2.csv",header = TRUE)
returns_daily3 <- read.csv("RETURNS_DAILY3.csv",header = TRUE)
returns_daily <- suppressWarnings(rbind(returns_daily1,returns_daily2,returns_daily3))

# load pre ratio sorted data:
setwd('..')
load(file="pe_ratio_ibes_crsp_ff_rp_filtered.RData")
setwd(wd)
ticker <- unique(pe_ratio_ibes_crsp_ff$ticker)
data_daily_firm <- data.frame()
for (i in seq(length(ticker))){
  i_ticker <- as.character(ticker[i])
  cat(paste0("sorting ticker: ",i_ticker, " percent left: ", round((length(ticker)-i)/length(ticker)*100,2), "% ", "\n"))
  i_pe_ratio_ibes_crsp_ff <- pe_ratio_ibes_crsp_ff[pe_ratio_ibes_crsp_ff$ticker%in%i_ticker,]
  i_permno <- unique(i_pe_ratio_ibes_crsp_ff$crsp_permno)
  if (length(i_permno)==1){
    i_returns_daily <- returns_daily[returns_daily$PERMNO%in%i_permno,]
    i_return <- suppressWarnings(as.numeric(as.character(i_returns_daily$RETX)))
    i_return_vol <- i_return
    i_return_vol[is.na(i_return_vol)] <- 0
    vol <- c(rep(NA,times=59), zoo::rollapply(i_return_vol^2, 60, sum))
    i_vol <- vol/60
    i_dates <- as.Date(paste0(as.character(substr(i_returns_daily$date,1,4)),"-",
                 as.character(substr(i_returns_daily$date,5,6)),"-",
                 as.character(substr(i_returns_daily$date,7,8))))
    i_pred_daily <- data.frame(rep(i_ticker,times=length(i_return)),i_dates,i_return,i_vol)
  
    tmp_idx <- which((i_pred_daily$i_dates>=date_range[1])*(i_pred_daily$i_dates<=date_range[2])==1)
    i_pred_daily <- i_pred_daily[tmp_idx, ]
  } else {
    i_returns_daily <- returns_daily[returns_daily$PERMNO%in%i_permno,]
    i_return <- suppressWarnings(as.numeric(as.character(i_returns_daily$RETX)))
    i_return_vol <- i_return
    i_return_vol[is.na(i_return_vol)] <- 0
    vol <- c(rep(NA,times=59), zoo::rollapply(i_return_vol^2, 60, sum))
    i_vol <- vol/60
    i_dates <- as.Date(paste0(as.character(substr(i_returns_daily$date,1,4)),"-",
                              as.character(substr(i_returns_daily$date,5,6)),"-",
                              as.character(substr(i_returns_daily$date,7,8))))
    if(any(diff(i_dates)<0))
      stop("multiple permno id but dates are not in order - check")
    
    i_pred_daily <- data.frame(rep(i_ticker,times=length(i_return)),i_dates,i_return,i_vol)
  }
  colnames(i_pred_daily) <- c("daily_ticker","daily_dates","daily_return","daily_vol")
  data_daily_firm <- rbind(data_daily_firm,i_pred_daily)
}
save(data_daily_firm,file = "data_daily_firm.RData")
