#=========================================================================
#``Panel Data Nowcasting: The Case of Price-Earnings Ratios ''
# by Andrii Babii, Ryan Ball, Eric Ghysels, Jonas Striaukas.
#-------------------------------------------------------------------------
# collect all data in one place
#=========================================================================

#=========================================================================
#      COLLECT FULL DATASET - main results
#=========================================================================


rm(list=ls())
# ------- SET WORKING DIRECTORY ------- #
if (! "rstudioapi" %in% installed.packages() ) install.packages("rstudioapi")
wd <- dirname(rstudioapi::getSourceEditorContext()$path)
setwd(wd)

if (! "lubridate" %in% installed.packages() ) install.packages("lubridate")
if (! "midasml" %in% installed.packages() ) install.packages("midasml") 
library(lubridate)
library(midasml)


# functions: 
# run ravenpack data check up file - match rp with the rest
source("check_rp_data.R")
source("crsp_ibes_pe_ratio_rp_filtered.R")
# load pe ratio data:
load(file="pe_ratio_ibes_crsp_ff_rp_filtered.RData")
# get full list of permno:

# set rp folder path:
rp_folder_path <- "RavenPack2020/rp_data_new"
# set firm level financial daily data path
daily_fin_firm_folder_path <- "Firmdaily/"
# set daily markets data path
daily_fin_market_folder_path <- "FREDdaily/"
# set monthly textual analysis data path
agg_text_folder_path <- "AggregateTextual/"
# set monthly macro data path
macro_folder_path <- "MacroMD/"

# load rp-permno link table:
load(file="RavenPack2020/rp_permno_link_table.RData")
# add index 1:number of rp series available
rp_table <- cbind(rp_table, c(1:dim(rp_table)[1]))
colnames(rp_table)[dim(rp_table)[2]] <- "id"
# load daily firm data:
load(paste0(daily_fin_firm_folder_path,.Platform$file.sep,"data_daily_firm.RData"))
# load daily market data:
load(paste0(daily_fin_market_folder_path,.Platform$file.sep,"daily_data.RData"))
# load monthly macro textual data:
load(paste0(agg_text_folder_path,.Platform$file.sep,"AggregateTextualData.RData"))
# which monthly macro textual data to use: 
text_vars <- c("Earnings","Earnings forecasts","Earnings losses","Recession","Revenue growth","Revised estimate")
# remove the other series:
txt.data.ind <- txt.data.ind[,colnames(txt.data.ind)%in%text_vars]
# bind in data frame:
txt_data <- data.frame(txt.date=txt.date,txt.data.ind=txt.data.ind)
# which monthly macro data to use:
fredmd_vars <- c("INDPRO","CPIAUCSL","UNRATE")

ticker <- as.character(unique(pe_ratio_ibes_crsp_ff$ticker))

# in-sample data dates:
dates_is <- list(monthEnd(seq.Date(as.Date("2000-10-01",origin = lubridate::origin),as.Date("2006-10-01",origin = lubridate::origin), by="quarter")),
                 monthEnd(seq.Date(as.Date("2000-11-01",origin = lubridate::origin),as.Date("2006-11-01",origin = lubridate::origin), by="quarter")),
                 monthEnd(seq.Date(as.Date("2000-12-01",origin = lubridate::origin),as.Date("2006-12-01",origin = lubridate::origin), by="quarter")))
# out-of-sample data dates:
dates_oos <- list(monthEnd(seq.Date(as.Date("2007-01-01",origin = lubridate::origin),as.Date("2017-04-01",origin = lubridate::origin), by="quarter")),
                  monthEnd(seq.Date(as.Date("2007-02-01",origin = lubridate::origin),as.Date("2017-05-01",origin = lubridate::origin), by="quarter")),
                  monthEnd(seq.Date(as.Date("2007-03-01",origin = lubridate::origin),as.Date("2017-06-01",origin = lubridate::origin), by="quarter")))
num_lags <- 4

dlags <- 252 # daily lags included
mlags <- 12 # monthly lags included
degree <- 3 # Legendre polynomial degree
w.d <- lb(degree=degree,a=0,b=1,jmax=dlags)/dlags
w.m <- lb(degree=degree,a=0,b=1,jmax=mlags)/mlags

# storage for data for each outcome variable:
data_1 <- data_2 <- data_3 <- list()

disp.flag <- FALSE
# whether to standardize variables:
standardize_covariates <- TRUE
# remove "RIG" firm due to erratic time series:
ticker <- ticker[which(!ticker=="RIG")]
source("support_functions.R")
for (i_firm in seq(length(ticker))){
  i_ticker <- ticker[i_firm]
  cat(paste0("sorting the firm: ", i_ticker, "; percentage left: ",round((1-i_firm/length(ticker))*100,2),"%", "\n"))
  # get i-th firm data:
  i_pe_ratio_ibes_crsp_ff <- pe_ratio_ibes_crsp_ff[pe_ratio_ibes_crsp_ff$ticker%in%i_ticker, ]
  # get i-th permno:
  i_permno <- unique(i_pe_ratio_ibes_crsp_ff$crsp_permno)
  if (length(i_permno)>1) # CCE - more than 1 permno
    message("more than 1 permno for a firm")
  # get fiscal quarter info
  i_fiscal <- unique(i_pe_ratio_ibes_crsp_ff$fiscal_start)
  if (length(i_fiscal)>1)
    stop("fiscal quarters info in ambiguous")
  # get firm-level news and financial data
  # news data:
  i_rp_table <- rp_table[rp_table$PERMNO%in%i_permno,]
  i_rp_id <- as.character(i_rp_table$RP_id)
  load(paste0(rp_folder_path,.Platform$file.sep,i_rp_id,".RData"))
  # financial data:
  i_data_daily_firm <- data_daily_firm[data_daily_firm$daily_ticker%in%i_ticker,]
  # get in-sample dates:
  i_dates_is <- dates_is[[i_fiscal]]
  # get ouf-of-sample dates:
  i_dates_oos <- dates_oos[[i_fiscal]]
  # number of oos periods:
  num_h <- length(i_dates_oos)
  # for each firm loop through out of sample periods
  data_firm_1 <- data_firm_2 <- data_firm_3 <- list()
  new_index <- new_index_noar <- NULL
  for (j in seq(num_h)){
    tmp_x1_in <- tmp_x2_in <- tmp_x3_in <- tmp_x1_out <- tmp_x2_out <- tmp_x3_out <- NULL
    forecasted_quarter <- i_dates_oos[j]
    
    if (j == 1){
      in_sample_quarters <- i_dates_is
    } else {
      in_sample_quarters <- c(i_dates_is,i_dates_oos[c(1:(j-1))])
    }
    
    # ------ from here we collect response and lagged values data ------#

    eps_date <- i_pe_ratio_ibes_crsp_ff$pe_ttm_actual_date  
    
    idx_in_sample <- which(eps_date%in%in_sample_quarters)
    idx_out_sample <- which(eps_date%in%forecasted_quarter)
    
    # compute log(P/E) ratio
    peit <- i_pe_ratio_ibes_crsp_ff$prc_announce[idx_in_sample]/i_pe_ratio_ibes_crsp_ff$eps[idx_in_sample]
    peit[peit<0.0] <- 1.0
    peit[peit==Inf] <- 1.0
    peit <- log(peit)
    
    peit_oos <- i_pe_ratio_ibes_crsp_ff$prc_announce[idx_out_sample]/i_pe_ratio_ibes_crsp_ff$eps[idx_out_sample]
    peit_oos[peit_oos<0.0] <- 1.0
    peit_oos[peit_oos==Inf] <- 1.0
    peit_oos <- log(peit_oos)
  
    # create proxy lags for peit where earnings are replaced by median nowcast
    peit_lag_tmp <- matrix(NA, nrow = length(peit)+1, ncol = num_lags)
    for (i in seq(num_lags)){
      idx_in_sample_lag <- which(eps_date%in%in_sample_quarters)-i
      idx_out_sample_lag <- max(which(eps_date%in%in_sample_quarters))-i+1
      tmp <- i_pe_ratio_ibes_crsp_ff$prc_announce[idx_in_sample_lag]/i_pe_ratio_ibes_crsp_ff$median_forecast_h0[idx_in_sample_lag]
      tmp <- c(tmp, i_pe_ratio_ibes_crsp_ff$prc_announce[idx_out_sample_lag]/i_pe_ratio_ibes_crsp_ff$median_forecast_h0[idx_out_sample_lag])
      tmp[tmp<0.0] <- 1.0
      tmp[tmp==Inf] <- 1.0
      tmp <- log(tmp)
      if (!all(tmp == 0)){
        tmp <- scale(tmp)
      }
      peit_lag_tmp[,i] <- tmp
    }

    peit_lag <- as.matrix(peit_lag_tmp[1:length(peit),])
    peit_lag_oos <- peit_lag_tmp[length(peit)+1,]
    # compute quarterly firm log returns
    rit <- i_pe_ratio_ibes_crsp_ff$prc_announce[idx_in_sample]/i_pe_ratio_ibes_crsp_ff$prc_announce[idx_in_sample-1]
    rit <- log(rit)
    rit_oos <- i_pe_ratio_ibes_crsp_ff$prc_announce[idx_out_sample]/i_pe_ratio_ibes_crsp_ff$prc_announce[idx_out_sample-1]
    rit_oos <- log(rit_oos)
    
    # compute log of earnings forecast errors
    eait <- i_pe_ratio_ibes_crsp_ff$eps[idx_in_sample]/i_pe_ratio_ibes_crsp_ff$median_forecast_h1[idx_in_sample]
    eait[eait<=0.0] <- 1.0 
    eait[is.na(eait)] <- 1.0
    eait[eait==Inf] <- 1.0
    eait <- log(eait)

    eait_oos <- i_pe_ratio_ibes_crsp_ff$eps[idx_out_sample]/i_pe_ratio_ibes_crsp_ff$median_forecast_h1[idx_out_sample]
    eait_oos[eait_oos<=0.0] <- 1.0 
    eait_oos[is.na(eait_oos)] <- 1.0
    eait_oos[eait_oos==Inf] <- 1.0
    eait_oos <- log(eait_oos) 
    
    # compute the log ratio of return at time t and the forecast of earning for t+1 given t
    peiteait1 <- i_pe_ratio_ibes_crsp_ff$prc_announce[max(idx_in_sample)]/i_pe_ratio_ibes_crsp_ff$median_forecast_h1[idx_out_sample]
    peiteait1[peiteait1<0.0] <- 1.0
    peiteait1 <- log(peiteait1)
      
    if (any(is.na(peiteait1))){
      stop("peiteait1 is nan")
    }
    
    # bind lags to covariates 
    tmp_x1_in <- cbind(tmp_x1_in,peit_lag)
    tmp_x1_out <- c(tmp_x1_out,peit_lag_oos)
    
    # tmp_x2_in <- cbind(tmp_x2_in,y2_lag)
    # tmp_x2_out <- c(tmp_x2_out,y2_lag_oos)
    #   
    # tmp_x3_in <- cbind(tmp_x3_in,y3_lag)
    # tmp_x3_out <- c(tmp_x3_out,y3_lag_oos)
    
    if (j == 1)
      new_index <- c(new_index,rep(1, times = num_lags))
    
    # ----- from here we collect covariates ------#
    # ---- firm level textual data
    sent_data <- sent_data[,c(1,2,3,4,5,6)]
    sent_data <- as.data.frame(sent_data)
    rp_date <- as.Date(sent_data$DATE)
    # sort dates
    rp_ref_date_in_h0 <- as.Date(dateMatch(in_sample_quarters,rp_date),origin = lubridate::origin)
    rp_ref_date_all_h0 <- c(rp_ref_date_in_h0,forecasted_quarter)
    
    rp_ref_date_all_h0 <- c(monthEnd(rp_ref_date_all_h0[1] %m+% -months(3)),rp_ref_date_all_h0)
    sent_data_series <- sent_data[,-1]
    var_names <- colnames(sent_data_series)
    tmp_names <- NULL
    for (j_rp_id in seq(5)){
      rp_series <- c(as.numeric(as.matrix(sent_data_series[,j_rp_id])))
      rp_series[is.nan(rp_series)]<-0 # impute zero values for NaNs
      rp_series[is.na(rp_series)]<-0
      # sort names
      i_var_name <- var_names[j_rp_id]
      poly_names <- NULL
      for(i in 1:(degree+1)){poly_names <- c(poly_names, paste0(i_var_name,"-poly-",i-1))}
      tmp_names <- c(tmp_names,poly_names)
      
      last_date <- rp_ref_date_all_h0[length(rp_ref_date_all_h0)]
      idx_last_date <- max(which(rp_date<=last_date))
      if(rp_date[idx_last_date]<last_date){
        last_date <- rp_date[idx_last_date+1]
      }
      idx <- rp_date <= last_date
      rp_series_h0 <- rp_series[idx]
      if (standardize_covariates)
        rp_series_h0 <- as.numeric(scale(rp_series_h0,center = TRUE,scale = TRUE))
      rp_date_dates_h0 <- rp_date[idx]
      
      rp_tmp_h0 <- mixed_freq_data_single(rp_ref_date_all_h0,rp_series_h0,rp_date_dates_h0,dlags,0,rp_ref_date_all_h0[1],rp_ref_date_all_h0[length(rp_ref_date_all_h0)-1],disp.flag=disp.flag)
      
      tmp_x1_in <- cbind(tmp_x1_in,rp_tmp_h0$est.x%*%w.d)
      tmp_x1_out <- c(tmp_x1_out,rp_tmp_h0$out.x%*%w.d)
      
      tmp_x2_in <- cbind(tmp_x2_in,rp_tmp_h0$est.x%*%w.d)
      tmp_x2_out <- c(tmp_x2_out,rp_tmp_h0$out.x%*%w.d)
      
      tmp_x3_in <- cbind(tmp_x3_in,rp_tmp_h0$est.x%*%w.d)
      tmp_x3_out <- c(tmp_x3_out,rp_tmp_h0$out.x%*%w.d)
      
      # adding indices to group covariates:
      if (j == 1)
        new_index <- c(new_index, max(new_index) + rep(1,degree+1))

      if (j == 1)
        new_index_noar <- c(new_index_noar,rep(1, times = degree+1))
    }
    
    # ---- firm level financial data
    firm_fin_dates <- as.Date(i_data_daily_firm$daily_dates)
    # sort dates
    firm_fin_ref_date_in_h0 <- as.Date(dateMatch(in_sample_quarters,firm_fin_dates),origin = lubridate::origin)
    firm_fin_ref_date_all_h0 <- c(firm_fin_ref_date_in_h0,forecasted_quarter)

    firm_fin_ref_date_all_h0 <- c(monthEnd(firm_fin_ref_date_all_h0[1] %m+% -months(3)),firm_fin_ref_date_all_h0)
    firm_fin_data_series <- i_data_daily_firm[,c(3,4)]
    var_names <- c("firm-return","firm-vola")
    for (j_firm_fin_id in seq(2)){
      firm_fin <- c(as.numeric(as.matrix(firm_fin_data_series[,j_firm_fin_id])))
      firm_fin[is.nan(firm_fin)]<-0 # impute zero values for NaNs
      firm_fin[is.na(firm_fin)]<-0
      # sort names
      i_var_name <- var_names[j_firm_fin_id]
      poly_names <- NULL
      for(i in 1:(degree+1)){poly_names <- c(poly_names, paste0(i_var_name,"-poly-",i-1))}
      tmp_names <- c(tmp_names,poly_names)
      
      
      last_date <- firm_fin_ref_date_all_h0[length(firm_fin_ref_date_all_h0)]
      idx_last_date <- max(which(firm_fin_dates<=last_date))
      if(firm_fin_dates[idx_last_date]<last_date){
        last_date <- firm_fin_dates[idx_last_date+1]
      }
      idx <- firm_fin_dates <= last_date
      firm_fin_h0 <- firm_fin[idx]
      if (standardize_covariates)
        firm_fin_h0 <- as.numeric(scale(firm_fin_h0, center = TRUE, scale = TRUE))
      firm_fin_dates_h0 <- firm_fin_dates[idx]
      
      firm_fin_tmp_h0 <- mixed_freq_data_single(firm_fin_ref_date_all_h0,firm_fin_h0,firm_fin_dates_h0,dlags,0,firm_fin_ref_date_all_h0[1],firm_fin_ref_date_all_h0[length(firm_fin_ref_date_all_h0)-1],disp.flag=disp.flag)
      
      tmp_x1_in <- cbind(tmp_x1_in,firm_fin_tmp_h0$est.x%*%w.d)
      tmp_x1_out <- c(tmp_x1_out,firm_fin_tmp_h0$out.x%*%w.d)
      
      tmp_x2_in <- cbind(tmp_x2_in,firm_fin_tmp_h0$est.x%*%w.d)
      tmp_x2_out <- c(tmp_x2_out,firm_fin_tmp_h0$out.x%*%w.d)
      
      tmp_x3_in <- cbind(tmp_x3_in,firm_fin_tmp_h0$est.x%*%w.d)
      tmp_x3_out <- c(tmp_x3_out,firm_fin_tmp_h0$out.x%*%w.d)
      
      
      # adding indices to group covariates:
      if (j == 1){
        new_index <- c(new_index, max(new_index) + rep(1,degree+1))
        new_index_noar <- c(new_index_noar, max(new_index_noar) + rep(1,degree+1))
      }
    }
    
    # ---- market financial data
    fin_dates <- as.Date(daily_data$DATE)
    # sort dates
    fin_ref_date_in_h0 <- as.Date(dateMatch(in_sample_quarters,fin_dates),origin = lubridate::origin)
    fin_ref_date_all_h0 <- c(fin_ref_date_in_h0,forecasted_quarter)

    fin_ref_date_all_h0 <- c(monthEnd(fin_ref_date_all_h0[1] %m+% -months(3)),fin_ref_date_all_h0)
    fin_data_series <- daily_data[,-1]
    
    var_names <- colnames(fin_data_series)
    
    for (j_fin_id in seq(9)){
      fin <- c(as.numeric(as.matrix(fin_data_series[,j_fin_id])))
      fin[is.na(fin)]<-0 # impute zero values for NaNs
      # sort names
      i_var_name <- var_names[j_fin_id]
      poly_names <- NULL
      for(i in 1:(degree+1)){poly_names <- c(poly_names, paste0(i_var_name,"-poly-",i-1))}
      tmp_names <- c(tmp_names,poly_names)
      
      last_date <- fin_ref_date_all_h0[length(fin_ref_date_all_h0)]
      idx_last_date <- max(which(fin_dates<=last_date))
      if(fin_dates[idx_last_date]<last_date){
        last_date <- fin_dates[idx_last_date+1]
      }
      idx <- fin_dates <= last_date
      fin_h0 <- fin[idx]
      if (standardize_covariates)
        fin_h0 <- as.numeric(scale(fin_h0, center = TRUE, scale = TRUE))
      fin_dates_h0 <- fin_dates[idx]
      
      fin_tmp_h0 <- mixed_freq_data_single(fin_ref_date_all_h0,fin_h0,fin_dates_h0,dlags,0,fin_ref_date_all_h0[1],fin_ref_date_all_h0[length(fin_ref_date_all_h0)-1],disp.flag=disp.flag)
      
      tmp_x1_in <- cbind(tmp_x1_in,fin_tmp_h0$est.x%*%w.d)
      tmp_x1_out <- c(tmp_x1_out,fin_tmp_h0$out.x%*%w.d)
      
      tmp_x2_in <- cbind(tmp_x2_in,fin_tmp_h0$est.x%*%w.d)
      tmp_x2_out <- c(tmp_x2_out,fin_tmp_h0$out.x%*%w.d)
      
      tmp_x3_in <- cbind(tmp_x3_in,fin_tmp_h0$est.x%*%w.d)
      tmp_x3_out <- c(tmp_x3_out,fin_tmp_h0$out.x%*%w.d)
      
      # adding indices to group covariates:
      if (j == 1){
        new_index <- c(new_index, max(new_index) + rep(1,degree+1))
        new_index_noar <- c(new_index_noar, max(new_index_noar) + rep(1,degree+1))
      }
    }
    
    # ---- monthly macro textual data
    text_dates <- as.Date(txt_data$txt.date)
    text_dates <- monthEnd(text_dates)
    text_ref_date_in_h0 <- as.Date(dateMatch(in_sample_quarters,text_dates),origin = lubridate::origin)
    text_ref_date_all_h0 <- c(text_ref_date_in_h0,forecasted_quarter)
    
    text_ref_date_all_h0 <- c(monthEnd(text_ref_date_all_h0[1] %m+% -months(3)),text_ref_date_all_h0)
    text_data_series <- txt_data[,-1]
    
    for (j_text_id in seq(dim(text_data_series)[2])){
      text <- c(as.numeric(as.matrix(text_data_series[,j_text_id])))
      # sort names
      i_var_name <- text_vars[j_text_id]
      poly_names <- NULL
      for(i in 1:(degree+1)){poly_names <- c(poly_names, paste0(i_var_name,"-poly-",i-1))}
      tmp_names <- c(tmp_names,poly_names)
      
      
      idx <- text_dates <= text_ref_date_all_h0[length(text_ref_date_all_h0)]
      text_h0 <- text[idx]
      if (standardize_covariates)
        text_h0 <- as.numeric(scale(text_h0, center = TRUE, scale = TRUE))
      text_dates_h0 <- text_dates[idx]
      
      text_tmp_h0 <- mixed_freq_data_single(text_ref_date_all_h0,text_h0,text_dates_h0,mlags,0,text_ref_date_all_h0[1],text_ref_date_all_h0[length(text_ref_date_all_h0)-1],disp.flag=disp.flag)
      
      tmp_x1_in <- cbind(tmp_x1_in,text_tmp_h0$est.x%*%w.m)
      tmp_x1_out <- c(tmp_x1_out,text_tmp_h0$out.x%*%w.m)
      
      tmp_x2_in <- cbind(tmp_x2_in,text_tmp_h0$est.x%*%w.m)
      tmp_x2_out <- c(tmp_x2_out,text_tmp_h0$out.x%*%w.m)
      
      tmp_x3_in <- cbind(tmp_x3_in,text_tmp_h0$est.x%*%w.m)
      tmp_x3_out <- c(tmp_x3_out,text_tmp_h0$out.x%*%w.m)

      # adding indices to group covariates:
      if (j == 1){
        new_index <- c(new_index, max(new_index) + rep(1,degree+1))
        new_index_noar <- c(new_index_noar, max(new_index_noar) + rep(1,degree+1))
      }
    }
    
    
    # ---- monthly macro data
    macro_data_h0 <- read.csv(paste0(macro_folder_path,format(forecasted_quarter,"%Y-%m"),".csv"))
    macro_series_h0 <- macro_data_h0[,colnames(macro_data_h0)%in%fredmd_vars]
    macro_dates_h0 <- as.character(macro_data_h0[-1,1])
    macro_dates_h0 <- as.Date(macro_dates_h0,format="%m/%d/%Y")
    macro_dates_h0 <- monthEnd(macro_dates_h0)
    
    in_sample_quarters_md <- monthEnd(in_sample_quarters%m+% -months(1))
    forecasted_quarter_md <- monthEnd(forecasted_quarter%m+% -months(1))
    macro_ref_date_in_h0 <- as.Date(dateMatch(in_sample_quarters_md,macro_dates_h0),origin = lubridate::origin)
    macro_ref_date_all_h0 <- c(macro_ref_date_in_h0,forecasted_quarter_md)
    macro_ref_date_all_h0 <- c(monthEnd(macro_ref_date_all_h0[1] %m+% -months(3)),macro_ref_date_all_h0)
    
    # transformations of macro:
    t_codes <- as.numeric(macro_series_h0[1,])
    macro_series_h0 <- macro_series_h0[-1,]
    macro_series_h0 <- apply_transform(macro_series_h0,t_codes)
    
    for (j_macro_id in seq(dim(macro_series_h0)[2])){
      macro_h0 <- c(as.numeric(as.matrix(macro_series_h0[,j_macro_id])))
      i_var_name <- fredmd_vars[j_macro_id]
      poly_names <- NULL
      for(i in 1:(degree+1)){poly_names <- c(poly_names, paste0(i_var_name,"-poly-",i-1))}
      tmp_names <- c(tmp_names,poly_names)
      if (standardize_covariates)
        macro_h0 <- as.numeric(scale(macro_h0, center = TRUE, scale = TRUE))
      
      macro_tmp_h0 <- mixed_freq_data_single(macro_ref_date_all_h0,macro_h0,macro_dates_h0,mlags,0,macro_ref_date_all_h0[1],macro_ref_date_all_h0[length(macro_ref_date_all_h0)-1],disp.flag=disp.flag)
      
      tmp_x1_in <- cbind(tmp_x1_in,macro_tmp_h0$est.x%*%w.m)
      tmp_x1_out <- c(tmp_x1_out,macro_tmp_h0$out.x%*%w.m)
      
      tmp_x2_in <- cbind(tmp_x2_in,macro_tmp_h0$est.x%*%w.m)
      tmp_x2_out <- c(tmp_x2_out,macro_tmp_h0$out.x%*%w.m)
      
      tmp_x3_in <- cbind(tmp_x3_in,macro_tmp_h0$est.x%*%w.m)
      tmp_x3_out <- c(tmp_x3_out,macro_tmp_h0$out.x%*%w.m)
      # adding indices to group covariates:
      if (j == 1){
        new_index <- c(new_index, max(new_index) + rep(1,degree+1))
        new_index_noar <- c(new_index_noar, max(new_index_noar) + rep(1,degree+1))
      }
    }
    
    name_lags <- NULL
    for (qqq in seq(num_lags)) { name_lags <- c(name_lags, paste0("lag-",qqq))}
    
    tmp_names <- c(name_lags,tmp_names)
    colnames(tmp_x1_in) <- tmp_names
    colnames(tmp_x2_in) <- tmp_names[-c(1:4)]
    colnames(tmp_x3_in) <- tmp_names[-c(1:4)]
    # store quarter data for each firm:
    data_firm_1[[j]] <- list(y = peit, y_oos = peit_oos, x_in = tmp_x1_in, x_out = tmp_x1_out, observed = peiteait1, new_index = new_index)
    data_firm_2[[j]] <- list(y = rit, y_oos = rit_oos, x_in = tmp_x2_in, x_out = tmp_x2_out, new_index = new_index_noar)
    data_firm_3[[j]] <- list(y = eait, y_oos = eait_oos, x_in = tmp_x3_in, x_out = tmp_x3_out, new_index = new_index_noar)
    #tmp_consensus_info_h1
  } #end of quarter loop
  
  data_1[[i_firm]] <- list(data_firm=data_firm_1,ff_class=unique(i_pe_ratio_ibes_crsp_ff$ff_10),ticker=i_ticker)
  data_2[[i_firm]] <- list(data_firm=data_firm_2,ff_class=unique(i_pe_ratio_ibes_crsp_ff$ff_10),ticker=i_ticker)
  data_3[[i_firm]] <- list(data_firm=data_firm_3,ff_class=unique(i_pe_ratio_ibes_crsp_ff$ff_10),ticker=i_ticker)
  
} # end of firm loop 
save(data_1,data_2,data_3,standardize_covariates,file = "data_full_midas.RData")



