transform_dt <- function(x,tcode){
  t <- length(x)
  small <- 1e-9
  y <- rep(NA, t) 
  if(tcode==1) { # Level (i.e. no transformation): x(t)
    y = x
  }
  if(tcode==2){ # First difference: x(t)-x(t-1)
    y[-1] <- x[-1] - x[-t]
  }
  if(tcode==3){ # Second difference: (x(t)-x(t-1))-(x(t-1)-x(t-2))
    y[-c(1,2)] <- x[-c(1,2)] - 2*x[-c(1,t)] + x[-c(t-1,t)]
  } 
  if(tcode==4){ # Natural log: ln(x)
    x[abs(x)<small] <- NA
    y <- log(x)
  }  
  if(tcode==5){ # First difference of natural log: ln(x)-ln(x-1)
    x[abs(x)<small] <- NA
    x <- log(x)
    y[-1] <- x[-1] - x[-t]
  }   
  if(tcode==6){ # Second difference of natural log: ln(x)-2*ln(x-1)+ln(x-2)
    x[abs(x)<small] <- NA
    x <- log(x)
    y[-c(1,2)] <- x[-c(1,2)] - 2*x[-c(1,t)] + x[-c(t-1,t)]
  }    
  if(tcode==7){ # First difference of percent change: (x(t)/x(t-1)-1)-(x(t-1)/x(t-2)-1)
    x[abs(x)<small] <- NA
    y.tmp <- y
    y.tmp[-1] <- (x[-1]-x[-t])/x[-t]
    y[-c(1,2)] <- y.tmp[-c(1,2)] - y.tmp[-c(1,t)] 
  } 
  return(y)
}

apply_transform <- function(data.in,tcode.all){
  data.in <- as.matrix(data.in)
  data.out <- matrix(data=NA, nrow = nrow(data.in), ncol = ncol(data.in))
  colnames(data.out) <- colnames(data.in)
  n <- dim(data.in)[2]
  for (i in 1:n){
    tmp <-  transform_dt(data.in[,i],tcode.all[i])
    data.out[,i] <- tmp
  }
  return(data.out)
}