# load data: 
load("data_read.RData")
rm(ibes) # IBES already sorted
# load sorted IBES data:
load("ibes_sorted.RData") 
# read in daily crsp prices 
#crsp_daily <- read.csv("prices-crsp-daily.csv")
load("crsp_daily.RData")
ibes_ticker <- ibes_sorted$ticker
ibes_ticker <- unique(ibes_ticker)

char2date <- function(charvec){
  # purpose: convert character date vector of format yyyymmdd (e.g. 20200331) to date format
  n <- length(charvec)
  datevec <- numeric(n)
  internal <- function(dt){
    dt <- as.Date(paste0(substr(dt,1,4),"-",substr(dt,5,6),"-",substr(dt,7,8)))
    dt
  }
  datevec <- as.Date(apply(as.matrix(charvec),1,internal),origin = lubridate::origin)
  datevec
}


dates <- list(monthEnd(seq.Date(as.Date("1999-01-01",origin = lubridate::origin),as.Date("2017-04-01",origin = lubridate::origin), by="quarter")),
              monthEnd(seq.Date(as.Date("1999-02-01",origin = lubridate::origin),as.Date("2017-05-01",origin = lubridate::origin), by="quarter")),
              monthEnd(seq.Date(as.Date("1999-03-01",origin = lubridate::origin),as.Date("2017-06-01",origin = lubridate::origin), by="quarter")))

sort_crsp_announce <- function(crsp,crsp_daily,ibes_crsp_link,i_ibes_ticker,dates_eps,dates_eps_announce){
  
  sort_dates <- function(date_vec){
    # purpose: to sort date vectors
    # Missing means the latest date is up to date:
    date_vec_out <- last_date <- NULL
    if (date_vec[length(date_vec)]==""){
      date_vec <- date_vec[-length(date_vec)]
      last_date <- as.Date("2020-03-14")
    }
    if (length(date_vec)!=0){
      dates_days <- substr(date_vec,1,2)
      dates_months <- substr(date_vec,3,5)
      dates_years <- substr(date_vec,6,nchar(date_vec))
      tmp <- NULL
      for (j in seq(length(dates_months))){
        if(dates_months[j]=="JAN") {tmp[j] <- 1}
        if(dates_months[j]=="FEB") {tmp[j] <- 2}
        if(dates_months[j]=="MAR") {tmp[j] <- 3}
        if(dates_months[j]=="APR") {tmp[j] <- 4}
        if(dates_months[j]=="MAY") {tmp[j] <- 5}
        if(dates_months[j]=="JUN") {tmp[j] <- 6}
        if(dates_months[j]=="JUL") {tmp[j] <- 7}
        if(dates_months[j]=="AUG") {tmp[j] <- 8}
        if(dates_months[j]=="SEP") {tmp[j] <- 9}
        if(dates_months[j]=="OCT") {tmp[j] <- 10}
        if(dates_months[j]=="NOV") {tmp[j] <- 11}
        if(dates_months[j]=="DEC") {tmp[j] <- 12}
      }
      dates_months <- as.character(tmp)
      date_vec_out <- numeric(length(date_vec))
      for (j in seq(length(date_vec))){
        j_dates_months <- dates_months[j]
        if (nchar(j_dates_months)==1){j_dates_months <- paste0("0",j_dates_months)}
        date_vec_out[j] <- as.Date(paste0(dates_years[j],"-",j_dates_months,"-",dates_days[j]))
      }
    }
    if (!is.null(last_date))
      date_vec_out <- c(date_vec_out,last_date)
    
    date_vec_out <- as.Date(date_vec_out,origin = lubridate::origin)
    date_vec_out
  }
  
  
  # get ibes and crsp link table for i_ibes_ticker company:
  
  i_link_table <- ibes_crsp_link[ibes_crsp_link$TICKER%in%i_ibes_ticker,]
  start_dates <- sort_dates(as.character(i_link_table$sdate))
  end_dates <- sort_dates(as.character(i_link_table$edate))
  tmp_idx <- NULL
  for (j in seq(length(end_dates))){
    tmp_idx[j] <- end_dates[j]>dates_eps[1]
  }
  start_dates <- start_dates[tmp_idx]
  end_dates <- end_dates[tmp_idx]
  i_link_table <- i_link_table[tmp_idx,]
  
  tmp_idx <- NULL
  for (j in seq(length(start_dates))){
    tmp_idx[j] <- start_dates[j]<dates_eps[length(dates_eps)]
  }
  start_dates <- start_dates[tmp_idx]
  end_dates <- end_dates[tmp_idx]
  i_link_table <- i_link_table[tmp_idx,]
  
  skip <- FALSE
  crsp_price_announce <- crsp_price <- crsp_permno <- NULL
  
  if (all(i_link_table$SCORE==1)){ # ensures that CRSP and IBES match was exact
    crsp_price_announce <- crsp_price <- crsp_permno <- rep(NA,length(dates_eps))
    
    if (length(unique(i_link_table$PERMNO))==1){
      i_permno <- unique(i_link_table$PERMNO)
      i_crsp <- crsp[crsp$PERMNO%in%i_permno,]
      i_crsp_daily <- crsp_daily[which(crsp_daily$PERMNO%in%i_permno),]

      i_crsp_dates <- char2date(i_crsp$date)
      i_crsp_daily_dates <- char2date(i_crsp_daily$date)
      # make end of month to match IBES dates (means: if last day of the month is not trading day we take the last day within the month that was a trading day)
      i_crsp_dates <- monthEnd(i_crsp_dates)
      i_idx_crsp <- which(i_crsp_dates%in%dates_eps)
      i_idx_crsp_announce <- which(i_crsp_daily_dates%in%dates_eps_announce)
      if((length(i_idx_crsp_announce)!=length(dates_eps_announce))){
        tmp_dates_eps_announce <- dates_eps_announce
        dates_not_found <- dates_eps_announce[!dates_eps_announce%in%i_crsp_daily_dates[i_idx_crsp_announce]]
        for (i_dnf in seq(length(dates_not_found))){
          if(!is.finite(max(which(i_crsp_daily_dates<=dates_not_found[i_dnf]))))
            stop()
          tmp_dates_eps_announce[which(tmp_dates_eps_announce%in%dates_not_found[i_dnf])] <- i_crsp_daily_dates[max(which(i_crsp_daily_dates<=dates_not_found[i_dnf]))]
        }
        i_idx_crsp_announce <- which(i_crsp_daily_dates%in%tmp_dates_eps_announce)
      }
      if(any(diff(i_crsp_dates[i_idx_crsp])==0)){
        idx <- !duplicated(i_crsp_dates[i_idx_crsp],fromLast = TRUE)
        i_idx_crsp <- i_idx_crsp[idx]
      }
      if ((length(i_idx_crsp)!=length(dates_eps))+(length(i_idx_crsp_announce)!=length(dates_eps_announce))!=0){
        stop(paste0("crsp sorting step, lenghts of index vectors dont match"))
        skip <- TRUE
      } else {
        crsp_price <- abs(i_crsp$PRC[i_idx_crsp])/i_crsp$CFACPR[i_idx_crsp]
        crsp_permno <- rep(i_permno,length(i_idx_crsp))
        crsp_price_announce <- abs(i_crsp_daily$PRC[i_idx_crsp_announce])/i_crsp_daily$CFACPR[i_idx_crsp_announce]
      }
    } 
    
    if  (length(unique(i_link_table$PERMNO))>1){
    for (i in seq(dim(i_link_table)[1])){
      i_permno <- i_link_table$PERMNO[i]
      i_start_dates <- start_dates[i]
      i_end_dates <- end_dates[i]
      if (i > 1){
        if (i_start_dates > end_dates[i-1]){
          # if start date of new permno creates a gap, we take start date as the a previous end date
          i_start_dates <- end_dates[i-1]
        }
      }
      idx_1 <- dates_eps>=i_start_dates
      idx_2 <- dates_eps<=i_end_dates
      idx_final <- which((idx_1*idx_2)==1)
      i_dates <- dates_eps[idx_final]
      
      idx_1 <- dates_eps_announce>=i_start_dates
      idx_2 <- dates_eps_announce<=i_end_dates
      idx_final_daily <- which((idx_1*idx_2)==1)
      i_dates_announce <- dates_eps_announce[idx_final_daily]
      
      # get prices for i_permno:
      i_crsp <- crsp[crsp$PERMNO%in%i_permno,]
      i_crsp_daily <- crsp_daily[which(crsp_daily$PERMNO%in%i_permno),]
      # get crsp dates: 
      # note: crsp dates are daily and monthly, while ibes quarterly
      i_crsp_dates <- char2date(i_crsp$date)
      i_crsp_daily_dates <- char2date(i_crsp_daily$date)
      # make end of month to match IBES dates (means: if last day of the month is not trading day we take the last day within the month that was a trading day)
      i_crsp_dates <- monthEnd(i_crsp_dates)
      i_idx_crsp <- which(i_crsp_dates%in%i_dates)
      i_idx_crsp_announce <- which(i_crsp_daily_dates%in%i_dates_announce)
      if(any(diff(i_crsp_dates[i_idx_crsp])==0)){
        idx <- !duplicated(i_crsp_dates[i_idx_crsp],fromLast = TRUE)
        i_idx_crsp <- i_idx_crsp[idx]
      }
      if((length(i_idx_crsp_announce)!=length(i_dates_announce))){
        tmp_dates_eps_announce <- i_dates_announce
        dates_not_found <- i_dates_announce[!i_dates_announce%in%i_crsp_daily_dates[i_idx_crsp_announce]]
        for (i_dnf in seq(length(dates_not_found))){
          tmp_idx <- suppressWarnings(max(which(i_crsp_daily_dates<=dates_not_found[i_dnf])))
          if(!is.finite(tmp_idx)){
            skip <- TRUE
          } else {
            tmp_dates_eps_announce[which(tmp_dates_eps_announce%in%dates_not_found[i_dnf])] <- i_crsp_daily_dates[tmp_idx]
            i_idx_crsp_announce <- which(i_crsp_daily_dates%in%tmp_dates_eps_announce)
          }
        }
      }
      # note: we take absolute value because: prices are taken as the end of the day price; 
      # if the price is non exsiting, crsp takes the average of bid & ask prices, and 
      # makes the PRC negative to indicate that it is the average of bid/ask. we use both whichever is available
      if(length(idx_final_daily)!=length(i_idx_crsp_announce))
        #stop()
      if (length(idx_final)!=length(i_idx_crsp) || length(idx_final_daily)!=length(i_idx_crsp_announce)){
        #stop(paste0("crsp sorting step, lenghts of index vectors dont match"))
        message(paste0("crsp sorting step, lenghts of index vectors dont match"))
        skip <- TRUE
      } else {
        # compute price adjusted for splits and dividends (divide by Cumulative Factor to Adjust Price)
        crsp_price[idx_final] <- abs(i_crsp$PRC[i_idx_crsp])/i_crsp$CFACPR[i_idx_crsp]
        crsp_permno[idx_final] <- rep(i_permno,length(idx_final))
        crsp_price_announce[idx_final_daily] <- abs(i_crsp_daily$PRC[i_idx_crsp_announce])/i_crsp_daily$CFACPR[i_idx_crsp_announce]
      }
    }
    }
    
  }
  if (length(unique(i_link_table$PERMNO)) == 1){
    if (is.na(unique(i_link_table$PERMNO))){
      crsp_price <- NA
      crsp_permno <- NA
      crsp_price_announce <- NA
    }
  }
  if (skip){
    crsp_price <- NA
    crsp_permno <- NA
    crsp_price_announce <- NA
  }
  return(list(crsp_price=crsp_price,crsp_permno=crsp_permno,crsp_price_announce=crsp_price_announce))
}

sort_crsp <- function(crsp,ibes_crsp_link,i_ibes_ticker,dates_eps){
  
  sort_dates <- function(date_vec){
    # purpose: to sort date vectors
    # Missing means the latest date is up to date:
    date_vec_out <- last_date <- NULL
    if (date_vec[length(date_vec)]==""){
      date_vec <- date_vec[-length(date_vec)]
      last_date <- as.Date("2020-03-14")
    }
    if (length(date_vec)!=0){
      dates_days <- substr(date_vec,1,2)
      dates_months <- substr(date_vec,3,5)
      dates_years <- substr(date_vec,6,nchar(date_vec))
      tmp <- NULL
      for (j in seq(length(dates_months))){
        if(dates_months[j]=="JAN") {tmp[j] <- 1}
        if(dates_months[j]=="FEB") {tmp[j] <- 2}
        if(dates_months[j]=="MAR") {tmp[j] <- 3}
        if(dates_months[j]=="APR") {tmp[j] <- 4}
        if(dates_months[j]=="MAY") {tmp[j] <- 5}
        if(dates_months[j]=="JUN") {tmp[j] <- 6}
        if(dates_months[j]=="JUL") {tmp[j] <- 7}
        if(dates_months[j]=="AUG") {tmp[j] <- 8}
        if(dates_months[j]=="SEP") {tmp[j] <- 9}
        if(dates_months[j]=="OCT") {tmp[j] <- 10}
        if(dates_months[j]=="NOV") {tmp[j] <- 11}
        if(dates_months[j]=="DEC") {tmp[j] <- 12}
      }
      dates_months <- as.character(tmp)
      date_vec_out <- numeric(length(date_vec))
      for (j in seq(length(date_vec))){
        j_dates_months <- dates_months[j]
        if (nchar(j_dates_months)==1){j_dates_months <- paste0("0",j_dates_months)}
        date_vec_out[j] <- as.Date(paste0(dates_years[j],"-",j_dates_months,"-",dates_days[j]))
      }
    }
    if (!is.null(last_date))
      date_vec_out <- c(date_vec_out,last_date)
    
    date_vec_out <- as.Date(date_vec_out,origin = lubridate::origin)
    date_vec_out
  }
  
  
  # get ibes and crsp link table for i_ibes_ticker company:
  
  i_link_table <- ibes_crsp_link[ibes_crsp_link$TICKER%in%i_ibes_ticker,]
  start_dates <- sort_dates(as.character(i_link_table$sdate))
  end_dates <- sort_dates(as.character(i_link_table$edate))
  tmp_idx <- NULL
  for (j in seq(length(end_dates))){
    tmp_idx[j] <- end_dates[j]>dates_eps[1]
  }
  start_dates <- start_dates[tmp_idx]
  end_dates <- end_dates[tmp_idx]
  i_link_table <- i_link_table[tmp_idx,]
  
  tmp_idx <- NULL
  for (j in seq(length(start_dates))){
    tmp_idx[j] <- start_dates[j]<dates_eps[length(dates_eps)]
  }
  start_dates <- start_dates[tmp_idx]
  end_dates <- end_dates[tmp_idx]
  i_link_table <- i_link_table[tmp_idx,]
  
  skip <- FALSE
  crsp_price <- crsp_permno <- NULL
  
  if (all(i_link_table$SCORE==1)){ # ensures that CRSP and IBES match was exact
    crsp_price <- crsp_permno <- rep(NA,length(dates_eps))
    
    if (length(unique(i_link_table$PERMNO))==1){
      i_permno <- unique(i_link_table$PERMNO)
      i_crsp <- crsp[crsp$PERMNO%in%i_permno,]
      
      i_crsp_dates <- char2date(i_crsp$date)
      # make end of month to match IBES dates (means: if last day of the month is not trading day we take the last day within the month that was a trading day)
      i_crsp_dates <- monthEnd(i_crsp_dates)
      i_idx_crsp <- which(i_crsp_dates%in%dates_eps)
      if(any(diff(i_crsp_dates[i_idx_crsp])==0)){
        idx <- !duplicated(i_crsp_dates[i_idx_crsp],fromLast = TRUE)
        i_idx_crsp <- i_idx_crsp[idx]
      }
      if ( (length(i_idx_crsp)!=length(dates_eps)) == TRUE ){
        message(paste0("crsp sorting step, lenghts of index vectors dont match"))
        skip <- TRUE
      } else {
        crsp_price <- abs(i_crsp$PRC[i_idx_crsp])/i_crsp$CFACPR[i_idx_crsp]
        crsp_permno <- rep(i_permno,length(i_idx_crsp))
      }
    } 
    
    if (length(unique(i_link_table$PERMNO))>1){
      for (i in seq(dim(i_link_table)[1])){
        i_permno <- i_link_table$PERMNO[i]
        i_start_dates <- start_dates[i]
        i_end_dates <- end_dates[i]
        if (i > 1){
          if (i_start_dates > end_dates[i-1]){
            # if start date of new permno creates a gap, we take start date as the a previous end date
            i_start_dates <- end_dates[i-1]
          }
        }
        idx_1 <- dates_eps>=i_start_dates
        idx_2 <- dates_eps<=i_end_dates
        idx_final <- which((idx_1*idx_2)==1)
        i_dates <- dates_eps[idx_final]
        
        
        # get prices for i_permno:
        i_crsp <- crsp[crsp$PERMNO%in%i_permno,]
        # get crsp dates: 
        # note: crsp dates are daily and monthly, while ibes quarterly
        i_crsp_dates <- char2date(i_crsp$date)
        # make end of month to match IBES dates (means: if last day of the month is not trading day we take the last day within the month that was a trading day)
        i_crsp_dates <- monthEnd(i_crsp_dates)
        i_idx_crsp <- which(i_crsp_dates%in%i_dates)
        if(any(diff(i_crsp_dates[i_idx_crsp])==0)){
          idx <- !duplicated(i_crsp_dates[i_idx_crsp],fromLast = TRUE)
          i_idx_crsp <- i_idx_crsp[idx]
        }
        # note: we take absolute value because: prices are taken as the end of the day price; 
        # if the price is non exsiting, crsp takes the average of bid & ask prices, and 
        # makes the PRC negative to indicate that it is the average of bid/ask. we use both whichever is available
        if (length(idx_final)!=length(i_idx_crsp)){
          #stop(paste0("crsp sorting step, lenghts of index vectors dont match"))
          message(paste0("crsp sorting step, lenghts of index vectors dont match"))
          skip <- TRUE
        } else {
          # compute price adjusted for splits and dividends (divide by Cumulative Factor to Adjust Price)
          crsp_price[idx_final] <- abs(i_crsp$PRC[i_idx_crsp])/i_crsp$CFACPR[i_idx_crsp]
          crsp_permno[idx_final] <- rep(i_permno,length(idx_final))
        }
      }
    }
    
  }
  
  
  if (length(unique(i_link_table$PERMNO)) == 1){
    if (is.na(unique(i_link_table$PERMNO))){
      crsp_price <- NA
      crsp_permno <- NA
    }
  }
  if (skip){
    crsp_price <- NA
    crsp_permno <- NA
  }
  return(list(crsp_price=crsp_price,crsp_permno=crsp_permno))
}


k <- d <- 0
verbose <- TRUE
ibes_ticker <- ibes_ticker[!is.na(ibes_ticker)]
data_clean <- data.frame()

for (i in seq(length(ibes_ticker))){
    i_ibes_ticker <- as.character(ibes_ticker[i])
    i_ibes_sorted <- ibes_sorted[ibes_sorted$ticker%in%i_ibes_ticker, ]
    i_fiscal_start <- unique(as.numeric(i_ibes_sorted$fiscal_start))
    i_fiscal_start <- i_fiscal_start[!is.na(i_fiscal_start)]
    if (length(i_fiscal_start)>1)
      stop(paste0("multiple fiscal calendar for a specific firm, ticker: ", i_ibes_ticker))
    i_date <- dates[[i_fiscal_start]]
    idx_1 <- i_date%in%i_ibes_sorted$eps_quarter_date
    idx_2 <- i_date%in%i_ibes_sorted$quarter_dates_h0
    idx_3 <- i_date%in%i_ibes_sorted$quarter_dates_h1
    if (all(idx_1) && all(idx_2) && all(idx_3)){
      k <- k + 1
      if (verbose)
        message(paste0("company ", i_ibes_ticker, " was screened to be without NA values in dates. total number of such companies are: ",k,"\n","checking eps and forecasts...","\n"))        
      
      tmp_idx_1 <- i_ibes_sorted$eps_quarter_date%in%i_date
      tmp_idx_2 <- i_ibes_sorted$quarter_dates_h0%in%i_date
      tmp_idx_3 <- i_ibes_sorted$quarter_dates_h1%in%i_date
      
      tmp_ticker <- as.character(i_ibes_sorted$ticker)[tmp_idx_1]
      tmp_cusip <- as.character(i_ibes_sorted$cusip)[tmp_idx_1]
      tmp_cname <- as.character(i_ibes_sorted$cname)[tmp_idx_1]
      tmp_eps <- as.numeric(i_ibes_sorted$eps)[tmp_idx_1]
      tmp_eps_announce_date <- as.Date(as.vector(i_ibes_sorted$eps_announce_date[tmp_idx_1]), origin=lubridate::origin)
      tmp_eps_quarter_date <- as.Date(as.vector(i_ibes_sorted$eps_quarter_date[tmp_idx_1]), origin=lubridate::origin)
      tmp_fiscal_start <- as.numeric(as.vector(i_ibes_sorted$fiscal_start[tmp_idx_1]))
      
      tmp_mean_forecast_h0 <- as.numeric(as.vector(i_ibes_sorted$mean_forecast_h0[tmp_idx_2]))
      tmp_median_forecast_h0 <- as.numeric(as.vector(i_ibes_sorted$median_forecast_h0[tmp_idx_2]))
      tmp_howmany_h0 <- as.numeric(as.vector(i_ibes_sorted$howmany_h0[tmp_idx_2]))
      tmp_quarter_dates_h0 <- as.Date(as.vector(i_ibes_sorted$quarter_dates_h0[tmp_idx_2]), origin=lubridate::origin)
      
      tmp_mean_forecast_h1 <- as.numeric(as.vector(i_ibes_sorted$mean_forecast_h1[tmp_idx_3]))
      tmp_median_forecast_h1 <- as.numeric(as.vector(i_ibes_sorted$median_forecast_h1[tmp_idx_3]))
      tmp_howmany_h1 <- as.numeric(as.vector(i_ibes_sorted$howmany_h1[tmp_idx_3]))
      tmp_quarter_dates_h1 <- as.Date(as.vector(i_ibes_sorted$quarter_dates_h1[tmp_idx_3]), origin=lubridate::origin)
      
      tmp <- sort_crsp_announce(crsp,crsp_daily,ibes_crsp_link,i_ibes_ticker,dates_eps=tmp_eps_quarter_date,dates_eps_announce = tmp_eps_announce_date)
      
      if (is.null(tmp$crsp_price)){
        stop("crps data sorting step did not succeed")
        tmp$crsp_price <- NA
        tmp$crsp_permno <- NA
        tmp$crsp_price_announce <- NA
      }

      
      tmp_crsp_prc_h0 <- tmp$crsp_price
      tmp_crsp_permno <- tmp$crsp_permno
      tmp_crsp_date_h0 <- tmp_eps_quarter_date
      tmp_crsp_prc_announce <- tmp$crsp_price_announce
      tmp_crsp_date_announce <- tmp_eps_announce_date

      tmp_crsp_date_h1 <- tmp_crsp_date_h0
      tmp_crsp_date_h1 <- tmp_crsp_date_h1 %m+% -months(1)
      tmp_crsp_date_h1 <- monthEnd(tmp_crsp_date_h1)
      
      tmp <- sort_crsp(crsp,ibes_crsp_link,i_ibes_ticker,dates_eps=tmp_crsp_date_h1)
      
      if (is.null(tmp$crsp_price)){
        #stop("crps data sorting step did not succeed")
        tmp$crsp_price <- NA
        tmp$crsp_permno <- NA
      }
      
      tmp_crsp_prc_h1 <- tmp$crsp_price
      
      if (all(!is.na(tmp_eps)) && all(!is.na(tmp_mean_forecast_h0)) && all(!is.na(tmp_mean_forecast_h1)) && all(!is.na(tmp_crsp_date_h0)) && all(!is.na(tmp_crsp_date_h1)) && all(!is.na(tmp_crsp_prc_h0)) && all(!is.na(tmp_crsp_prc_h1)) ) {
        d <- d + 1
        if (verbose)
          message(paste0("company ", i_ibes_ticker, " was screened to be without NA values. total number: ",d, "\n"))
          tmp_dataframe <- data.frame(ticker = tmp_ticker,cusip = tmp_cusip,cname = tmp_cname,crsp_permno = tmp_crsp_permno,fiscal_start = tmp_fiscal_start,
                                      quarter_date = tmp_eps_quarter_date,eps_announce_date = tmp_eps_announce_date,quarter_dates_h0 = tmp_quarter_dates_h0,quarter_dates_h1 = tmp_quarter_dates_h1,
                                      eps = tmp_eps,
                                      prc_announce = tmp_crsp_prc_announce,crsp_date_announce = tmp_crsp_date_announce,
                                      prc_h0 = tmp_crsp_prc_h0,crsp_date_h0 = tmp_crsp_date_h0,
                                      prc_h1 = tmp_crsp_prc_h1,crsp_date_h1 = tmp_crsp_date_h1,
                                      mean_forecast_h0 = tmp_mean_forecast_h0,median_forecast_h0 = tmp_median_forecast_h0,howmany_h0 = tmp_howmany_h0,
                                      mean_forecast_h1 = tmp_mean_forecast_h1,median_forecast_h1 = tmp_median_forecast_h1,howmany_h1 = tmp_howmany_h1)
          data_clean <- rbind(data_clean,tmp_dataframe)
      }
    }
}
ibes_crsp_data <- data_clean
save(ibes_crsp_data,file = "ibes_crsp_data.RData")
