# read data from excel spreadsheets:
# ibes <- read.csv("eps-wrds-new-ibes.csv")
# crsp <- read.csv("prices-crsp.csv")
# ibes_crsp_link <- read.csv("ibes-crsp-link.csv")
# save(ibes, crsp, ibes_crsp_link, file = "data_read.RData")

# load read data:
load("data_read.RData")

ibes_ticker <- ibes$TICKER
ibes_ticker <- unique(ibes_ticker)

# filter ibes tickers in the link table that are in ibes_ticker and get the filtered table:
ibes_crsp_link <- ibes_crsp_link[ibes_crsp_link$TICKER%in%ibes_ticker,]


char2date <- function(charvec){
  # purpose: convert character date vector of format yyyymmdd (e.g. 20200331) to date format
  n <- length(charvec)
  datevec <- numeric(n)
  internal <- function(dt){
    dt <- as.Date(paste0(substr(dt,1,4),"-",substr(dt,5,6),"-",substr(dt,7,8)))
    dt
  }
  datevec <- as.Date(apply(as.matrix(charvec),1,internal),origin = lubridate::origin)
  datevec
}

monthEnd <- function(x){
  lubridate::ceiling_date(x, "month") - lubridate::days(1)
}

quarter_diff <- function(date1,date2,fiscal_start){
  q1 <- quarter(date1,fiscal_start = fiscal_start)
  y1 <- year(date1)
  q2 <- quarter(date2,fiscal_start = fiscal_start)
  y2 <- year(date2)
  q_diff <-  q1 - q2 + (y1 - y2)*4
  q_diff
}

sort_company_ibes <- function(i_ibes, PDF = "D", horizons = c(6,7), verbose = FALSE){

  # inputs: i_ibes - filtered firm specific IBES data
  #         PDF - which eps to take: P-primar
  # screen by PDF:
  idx <- i_ibes$PDF%in%PDF
  i_ibes <- i_ibes[idx, ]
  # check most common forecasted fiscal quarters:
  forecast_dates_all <- char2date(i_ibes$FPEDATS)
  forecast_dates <- unique(forecast_dates_all)
  num_quart <- numeric(3)
  for (j in 1:3){
    num_quart[j] <- sum((lubridate::month(forecast_dates)-j)%%3==0)
  }
  fiscal_start <- which.max(num_quart)
  # fiscal_start = 1 -> January, April, July, October; 
  # fiscal_start = 2 -> February, May, August, November; 
  # fiscal_start = 3 -> March, June, September, December;
  # get data that have dates based on 'quarters' fiscal quarter:
  idx <- (lubridate::month(forecast_dates_all)-fiscal_start)%%3==0
  i_ibes <- i_ibes[idx, ]
  # first horizon: 
  i_ibes_h0 <- i_ibes[i_ibes$FPI==horizons[1],]
  # second horizon:
  i_ibes_h1 <- i_ibes[i_ibes$FPI==horizons[2],]
  if (length(horizons)>2)
    stop("only two horizons are currently implemented, current quarter and 1 quarter ahead")
  

  
  # ---- initialize spaces ---- #  
  eps <- eps_date <-  NULL
  
  quarter_dates_h0 <- mean_forecast_h0 <- median_forecast_h0 <- howmany_h0 <- NULL
  
  forecast_dates_h0 <- unique(i_ibes_h0$FPEDATS)
  ticker <- cusip <- cname <- NULL
  
  # ---- sort horizon 0 ---- #
  k <- 0
  for (j_date in seq(length(forecast_dates_h0))){

    j_forecast_date <- forecast_dates_h0[j_date]
    j_idx <- i_ibes_h0$FPEDATS%in%j_forecast_date
    j_i_ibes_h0 <- i_ibes_h0[j_idx,]
    
    # sort ticker, cusip and cname
    tmp_ticker <- as.character(unique(j_i_ibes_h0$TICKER))
    if(length(tmp_ticker)>1)
      stop("multiple ticker per filtered company per period.")
    ticker <- c(ticker,tmp_ticker)
    
    # taking cusip and cname as of ACTUAL publication
    tmp_cusip <- as.character(unique(j_i_ibes_h0$CUSIP))
    cusip <- c(cusip,tmp_cusip[length(tmp_cusip)])
    
    tmp_cname <- as.character(unique(j_i_ibes_h0$CNAME))
    cname <- c(cname,tmp_cname[length(tmp_cname)])
    
    j_i_actual <- unique(as.numeric(j_i_ibes_h0$ACTUAL))
    if (any(is.na(j_i_ibes_h0$ANNDATS_ACT))){
      j_i_actual <- NA
      j_i_actual_date <- NA
    } else {
      j_i_actual_date <- unique(char2date(j_i_ibes_h0$ANNDATS_ACT))
    }
    if (length(j_i_actual)>1){
      j_i_actual <- NA
      j_i_actual_date <- j_i_actual_date[1]
    }
    if (is.na(j_i_actual)){
      j_i_actual_date <- NA
    }
    j_forecast_date <- char2date(j_forecast_date)
    if (j_date==1){
      quarter_dates_h0 <- c(quarter_dates_h0,j_forecast_date)
    } else {
      tmp_date1 <- as.Date(j_forecast_date,origin = lubridate::origin)
      tmp_date2 <- as.Date(quarter_dates_h0[j_date+k-1],origin = lubridate::origin)
      date_diff <- quarter_diff(tmp_date1,tmp_date2,fiscal_start)
      if (date_diff > 1) {
        date_diff <- date_diff - 1
        for (i_date_diff in seq(date_diff)){
          eps <- c(eps,NA)
          eps_date <- c(eps_date,NA)
          mean_forecast_h0 <- c(mean_forecast_h0,NA)
          median_forecast_h0 <- c(median_forecast_h0,NA)
          howmany_h0 <- c(howmany_h0,NA)
          tmp <- as.Date(quarter_dates_h0[j_date+k-1],origin = lubridate::origin)
          tmp <- tmp %m+% months(3)
          quarter_dates_h0 <- c(quarter_dates_h0,tmp)
          ticker <- c(ticker,NA)
          cname <- c(cname,NA)
          cusip <- c(cusip,NA)
          k <- k + 1
        }
      } 
    }
    
    eps <- c(eps,j_i_actual)
    eps_date <- c(eps_date,j_i_actual_date)
    if (j_date!=1){
      quarter_dates_h0 <- c(quarter_dates_h0,j_forecast_date)
    }
    # forecasts:
    # get index such that all forecasts are made up to j_forecast_dates:
    tmp1 <- char2date(j_i_ibes_h0$ANNDATS)
    idx_tmp <- tmp1<=j_forecast_date
    j_i_ibes_h0 <- j_i_ibes_h0[idx_tmp,]
    
    # remove duplicates by keeping the most recent forecast:
    idx_tmp2 <- !duplicated(cbind(j_i_ibes_h0$ESTIMATOR,j_i_ibes_h0$ANALYS),fromLast = TRUE)
    j_i_ibes_h0 <- j_i_ibes_h0[idx_tmp2,]
    
    # compute mean forecast:
    mean_eps <- mean(as.numeric(j_i_ibes_h0$VALUE))
    # compute median forecast:
    median_eps <- median(as.numeric(j_i_ibes_h0$VALUE))
    
    mean_forecast_h0 <- c(mean_forecast_h0,mean_eps)
    median_forecast_h0 <- c(median_forecast_h0,median_eps)
    
    howmany_h0 <- c(howmany_h0,length(j_i_ibes_h0$VALUE))
    
    if (verbose){
      quarter_dates_h0 <- as.Date(quarter_dates_h0,origin = lubridate::origin)
      cat(paste(j_forecast_date," - j_forecast_date","\n"))
      cat(paste(quarter_dates_h0," - quarter_dates_h0","\n"))
      cat(paste(quarter_dates_h0[j_date+k]," - quarter_dates_h0[j_date+k]","\n"))
      cat(paste("--------------------------","\n"))
      cat(paste("\n"))
      #Sys.sleep(1)
    }
  }
  
  
  forecast_dates_h1 <- unique(i_ibes_h1$FPEDATS)
  
  # ---- initialize spaces ---- #  
  quarter_dates_h1 <- mean_forecast_h1 <- median_forecast_h1 <- howmany_h1 <- NULL
  
  # ---- sort horizon 1 ---- #
  k <- 0
  for (j_date in seq(length(forecast_dates_h1))){
    j_forecast_date <- forecast_dates_h1[j_date]
    j_idx <- i_ibes_h1$FPEDATS%in%j_forecast_date
    j_i_ibes_h1 <- i_ibes_h1[j_idx,]
    j_forecast_date <- char2date(j_forecast_date)
    if (j_date==1){
      quarter_dates_h1 <- c(quarter_dates_h1,j_forecast_date)
    } else {
      tmp_date1 <- as.Date(j_forecast_date,origin = lubridate::origin)
      tmp_date2 <- as.Date(quarter_dates_h1[j_date+k-1],origin = lubridate::origin)
      date_diff <- quarter_diff(tmp_date1,tmp_date2,fiscal_start)
      if (date_diff > 1) {
        #stop("")
        date_diff <- date_diff - 1
        for (i_date_diff in seq(date_diff)){
          mean_forecast_h1 <- c(mean_forecast_h1,NA)
          median_forecast_h1 <- c(median_forecast_h1,NA)
          howmany_h1 <- c(howmany_h1,NA)
          tmp <- as.Date(quarter_dates_h1[j_date+k-1],origin = lubridate::origin)
          tmp <- tmp %m+% months(3)
          quarter_dates_h1 <- c(quarter_dates_h1,tmp)
          k <- k + 1
        }
      } 
    }
    if (j_date!=1){
      quarter_dates_h1 <- c(quarter_dates_h1,j_forecast_date)
    }
    # forecasts:
    # get index such that all forecasts are made up to j_forecast_date - 1month
    tmp <- j_forecast_date %m+% -months(1)
    tmp <- monthEnd(tmp)
    tmp1 <- char2date(j_i_ibes_h1$ANNDATS)
    idx_tmp <- tmp1<=tmp
    j_i_ibes_h1 <- j_i_ibes_h1[idx_tmp,]
    
    # remove duplicates by keeping the most recent forecast:
    idx_tmp2 <- !duplicated(cbind(j_i_ibes_h1$ESTIMATOR,j_i_ibes_h1$ANALYS),fromLast = TRUE)
    j_i_ibes_h1 <- j_i_ibes_h1[idx_tmp2,]
    
    # compute mean forecast:
    mean_eps <- mean(as.numeric(j_i_ibes_h1$VALUE))
    # compute median forecast:
    median_eps <- median(as.numeric(j_i_ibes_h1$VALUE))
    
    mean_forecast_h1 <- c(mean_forecast_h1,mean_eps)
    median_forecast_h1 <- c(median_forecast_h1,median_eps)
    
    howmany_h1 <- c(howmany_h1,length(j_i_ibes_h1$VALUE))
    
    if (verbose){
      quarter_dates_h1 <- as.Date(quarter_dates_h1,origin = lubridate::origin)
      cat(paste(j_forecast_date," - j_forecast_date","\n"))
      cat(paste(quarter_dates_h1 ," - quarter_dates_h1","\n"))
      cat(paste(quarter_dates_h1[j_date+k]," - quarter_dates_h1[j_date+k]","\n"))
      cat(paste("--------------------------","\n"))
      cat(paste("\n"))
      #Sys.sleep(1)
    }
  }
  
  # sort dates
  quarter_dates_h0 <- as.Date(quarter_dates_h0,origin = lubridate::origin)
  quarter_dates_h1 <- as.Date(quarter_dates_h1,origin = lubridate::origin)
  
  eps_announce_date <- as.Date(eps_date,origin = lubridate::origin)
  eps_quarter_date <- quarter_dates_h0#char2date(unique(i_ibes_h0$FPEDATS))
  eps_quarter_date[is.na(eps)] <- NA
  
  fiscal_start <- rep(fiscal_start,times=length(eps))
  
  mean_forecast_h0[is.nan(mean_forecast_h0)] <- NA
  median_forecast_h0[is.nan(median_forecast_h0)] <- NA
  howmany_h0[howmany_h0==0] <- NA
  
  mean_forecast_h1[is.nan(mean_forecast_h1)] <- NA
  median_forecast_h1[is.nan(median_forecast_h1)] <- NA
  howmany_h1[howmany_h1==0] <- NA
  data_h0 <- data.frame(ticker=ticker,cusip=cusip,cname=cname,
                        eps=eps,eps_announce_date=eps_announce_date,eps_quarter_date=eps_quarter_date,fiscal_start=fiscal_start,
                        mean_forecast_h0=mean_forecast_h0,median_forecast_h0=median_forecast_h0,howmany_h0=howmany_h0,quarter_dates_h0=quarter_dates_h0)
  data_h1 <- data.frame(mean_forecast_h1=mean_forecast_h1,median_forecast_h1=median_forecast_h1,howmany_h1=howmany_h1,quarter_dates_h1=quarter_dates_h1)
  if (dim(data_h0)[1]>dim(data_h1)[1]){
    diff_rows <- dim(data_h0)[1]-dim(data_h1)[1]
    tmp <- matrix(NA,nrow=diff_rows,ncol=dim(data_h1)[2])
    colnames(tmp) <- colnames(data_h1)
    data_h1 <- rbind(data_h1,tmp)
  }
  if (dim(data_h0)[1]<dim(data_h1)[1]){
    diff_rows <- dim(data_h1)[1]-dim(data_h0)[1]
    tmp <- matrix(NA,nrow=diff_rows,ncol=dim(data_h0)[2]-3)
    tmp <- cbind(rep(ticker,times=diff_rows),rep(cusip,times=diff_rows),rep(cname,times=diff_rows),matrix(NA,nrow=diff_rows,ncol=dim(data_h0)[2]-3))
    colnames(tmp) <- colnames(data_h0)
    data_h0 <- rbind(data_h0,tmp)
  }
  # replaces NAs in ticker cusip and cnames
  idx <- is.na(data_h0$ticker)
  if (any(idx)){
    # get approximately cusip and cnames (might change through time) and exact ticker:
    tmp_replace <- as.matrix(data_h0[which(diff(which(!is.na(data_h0$ticker)))!=1),c(1,2,3)])
    tmp_howmany <- length(which(idx))
    tmp_howmany = 2
    if (tmp_howmany>1){
      tmp <- tmp_replace
      for (i in 1:(tmp_howmany-1)){
        tmp <- rbind(tmp,tmp_replace)
      }
      tmp_replace <- tmp
    }
    data_h0[idx,c(1,2,3)] <- tmp_replace
  }

  data <- data.frame(data_h0,data_h1)
  data
}



# # split sorting in two parts if your PC has <= 8GB ram:
ibes_sorted <- data.frame()
for (i in seq(9172)){
  i_ibes_ticker <- as.character(ibes_ticker[i])
  i_ibes_crsp_link <- ibes_crsp_link[ibes_crsp_link$TICKER%in%i_ibes_ticker,]
  cat(paste0("sorting firm: ", i, " ticker: ",i_ibes_ticker, "\n"))
  #  stop("not unique permno for unique ibes ticker")
  # get ith rows corresponding to i_ibes_ticker from the ibes dataset:
  i_ibes <- ibes[ibes$TICKER%in%i_ibes_ticker,]
  # sort ibes data:
  #tmp_ibes_sorted <- sort_company_ibes(i_ibes,PDF = 'D', horizon = c(6,7), verbose = TRUE)
  tmp_ibes_sorted <- suppressWarnings(tryCatch(sort_company_ibes(i_ibes,PDF = 'D', horizon = c(6,7), verbose = TRUE), error=function(e) NULL))
  ibes_sorted <- rbind(ibes_sorted,tmp_ibes_sorted)
}
save(ibes_sorted,file = "ibes_sorted_part_1.RData")

